#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-07


from gagospecs.base.base import SpecBase
from gagospecs.v1.base.field_mappings import FIELD_MAPPINGS
from gagospecs.v1.base.load_module import create_module
from gagospecs.v1.base.model import DataProperties
from gagospecs.v1.base.schema import GMDSchema, create_schema


class FieldValidator(SpecBase):
    """
    Field validator
    """
    def __init__(
            self,
            name: str,
            field_name: str
    ):
        super().__init__(name)
        self.field_name = field_name

    def validate(
            self,
            value
    ):
        pass


class Validator(SpecBase):
    """
    Spec validator base class
    """
    def __init__(
            self,
            name: str,
            init_field_validators: list
    ):
        super().__init__(name)
        self.field_validators: dict = dict()
        for field_validator in init_field_validators:
            if field_validator.field_name \
                    not in self.field_validators:
                self.field_validators[field_validator.field_name] = \
                    field_validator

    @staticmethod
    def validate(
            product_level: str,
            product_name: str,
            gmd: dict
    ) -> dict:
        """
        validate a given ghf model
        :param product_level
        :param product_name
        :param gmd:
        :return:
        """
        schema: GMDSchema = create_schema(
            product_level,
            product_name
        )
        return schema.validate(gmd)

    def validate_data(
            self,
            data_properties: DataProperties,
            gmd: dict
    ) -> dict:
        """
        Validate the data properties
        :param data_properties: data properties
        :param gmd: gago meta data dict object
        :return:
        """
        for k, v in self.field_validators:
            field_name: str = k
            field_validator: FieldValidator = v
            if hasattr(data_properties, field_name):
                field_validator.validate(getattr(data_properties, field_name))
        errors: dict = self._validate_data(data_properties, gmd)
        input_errors = dict()
        if errors:
            for k, v in errors.items():
                field_name: str = k
                error_msg: str = v
                if field_name in FIELD_MAPPINGS:
                    input_errors[FIELD_MAPPINGS[field_name]] = error_msg
                else:
                    self.logger.error(
                        'field name %s not in FIELD_MAPPING',
                        field_name
                    )
        return input_errors

    def _validate_data(
            self,
            data_properties: DataProperties,
            gmd: dict
    ) -> dict:
        raise NotImplementedError


class CoordinateSystemFieldValidator(FieldValidator):
    def __init__(self):
        super().__init__(
            'CoordinateSystemFieldValidator',
            'coordinate_system'
        )

    def validate(
            self,
            value
    ):
        if isinstance(value, (float, int)):
            raise ValueError('coordinate system input error')


def create_validator(
        product_level: str,
        product_type: str
) -> Validator or None:
    m = create_module(
        'validators',
        product_level,
        product_type
    )
    if m and hasattr(m, 'main'):
        return m.main()
    return None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from marshmallow import validates, \
    ValidationError, \
    post_load
from marshmallow.fields import Dict

from gagospecs.v1.base.schema import RasterGMDSchema, \
    GeoTIFFGMDSchema
from gagospecs.v1 import ProductTypeValues, \
    FileTypeValues, \
    RasterDataTypeValues, \
    CompressionValues
from gagospecs.v1.models.original.CropClassification \
    import CropClassificationModel
from gagospecs.v1.base.schema import post_load_object


class CropClassificationSchema(RasterGMDSchema, GeoTIFFGMDSchema):
    """
    Crop classification schema
    """
    classification = Dict(required=True)

    @validates('file_type')
    def validate_file_type(self, value):
        if value != FileTypeValues.GeoTIFF:
            raise ValidationError('作物分类栅格产品必须为GeoTIFF格式')

    @validates('product_type')
    def validate_product_type(self, value):
        if value != ProductTypeValues.CropClassification:
            raise ValidationError('作物分类栅格产品的product_type'
                                  '字段必须为"CropClassification"')

    @validates('data_type')
    def validate_data_type(self, value):
        if value != RasterDataTypeValues.Byte:
            raise ValidationError('作物分类栅格产品data_type字段必须为"Byte"')

    @validates('no_data')
    def validate_no_data(self, value):
        if value != 255:
            raise ValidationError('作物分类栅格产品no_data字段必须为"255"')

    @validates('compression')
    def validate_linear_factor(self, value):
        if value != CompressionValues.LZW:
            raise ValidationError('作物分类栅格产品产品compression字段必须为"LZW"')

    @validates('classification')
    def validate_classification(self, value):
        value: dict = value
        for k, v in value.items():
            if not isinstance(k, int) or k < 0 or k > 254:
                raise ValidationError('classification分类值错误')
            if not isinstance(v, str):
                raise ValidationError('classification分类值错误')

    @post_load
    def make_object(self, data: dict):
        return post_load_object(CropClassificationModel(), **data)


def main():
    return CropClassificationSchema()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from json import loads, JSONDecodeError

from gagospecs.v1.base.load_model import \
    load_level_and_type_from_gmd, \
    load_level_and_type_from_gmd_str
from gagospecs.v1.base.model import DataProperties
from gagospecs.v1.base.model import create_data_properties
from gagospecs.v1.base.validator import create_validator, \
    Validator


def get_data_properties(
        gmd_str: str,
) -> dict:
    """
    get a DataProperties object from a gmd str
    :param gmd_str: gmd str from gmd file or url
    :return: DataProperties object
    """
    product_level, product_type, errors = \
        load_level_and_type_from_gmd_str(
            gmd_str
        )
    if errors:
        return errors
    return create_data_properties(
        product_level,
        product_type
    )


def validate_data(
        data_properties: DataProperties,
        gmd_str: str,
) -> dict:
    """
    validate if data is correct and if data matches mata data
    :param gmd_str: gago meta data dict object, from json str
    :param data_properties: properties of data itself(GeoTIFF, GeoJSON...)
    :return:
    """
    try:
        gmd_dict: dict = loads(gmd_str)
    except JSONDecodeError:
        return {'gmd内容': 'Gago Meta Data 内容错误'}

    product_level, product_type, errors = \
        load_level_and_type_from_gmd(
            gmd_dict
        )
    if errors:
        return errors
    validator: Validator = create_validator(
        product_level,
        product_type
    )
    return validator.validate_data(
        data_properties,
        gmd_dict
    )



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-08


from gagospecs.v1.base.generator import GMDGenerator
from gagospecs.v1.models.original.NDVI import NDVIModel
from gagospecs.v1.schemas.original.NDVI import NDVISchema


class NDVIGMDGenerator(GMDGenerator):
    def __init__(self):
        super().__init__(
            'NDVIProductGenerator'
        )

    def generate(
            self,
            ndvi_model: NDVIModel
    ) -> (str, dict):
        schema: NDVISchema = NDVISchema()
        return schema.dumps(ndvi_model)


def main():
    return NDVIGMDGenerator()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from json import loads, JSONDecodeError

from gagospecs.v1.base.load_model import load_level_and_type_from_gmd
from gagospecs.v1.base.validator import create_validator, Validator


def validate_gmd(
        gmd_str: str
) -> dict:
    """
    Validate a gmd dict object loaded from a gmd file or url
    :param gmd_str:
    :return: errors. if it is empty, the gmd is right
    """
    try:
        gmd_dict: dict = loads(gmd_str)
    except JSONDecodeError:
        return {'GMD内容': 'Gago Meta Data 内容错误'}

    product_level, product_type, errors = \
        load_level_and_type_from_gmd(
            gmd_dict
        )
    if errors:
        return errors

    validator: Validator = create_validator(
        product_level,
        product_type
    )
    if not validator:
        return {
            'GMD内容':
                '"level"和"product_type字段可能有误，'
                '无法加载规格"'
        }

    return validator.validate(
        product_level,
        product_type,
        gmd_dict
    )



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-17


from gagospecs.base.base import SpecBase
from gagospecs.v1.base.load_module import create_module


class Converter(SpecBase):
    """
    Gago data product converter base class,
    support automatic conversion of some kind of products,
    for example, ndvi and classification
    """


def create_converter(
        product_level: str,
        product_type: str
) -> Converter or None:
    m = create_module(
        'converters',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main'):
            return m.main()
    return None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from gagospecs.v1.base.model import DataProperties, \
    CategoryValues
from gagospecs.v1.base.model import GeoTIFFRasterGMDModel, \
    RasterGMDModel
from gagospecs.v1 import ProductTypeValues, \
    FileTypeValues, \
    CoordinateSystemValues, \
    RasterDataTypeValues, \
    CompressionValues


class CropClassificationModel(RasterGMDModel, GeoTIFFRasterGMDModel):
    def __init__(self):
        super(CropClassificationModel, self).__init__()
        self.name = 'CropClassification'
        self.description = 'CropClassification'
        self.category = CategoryValues.Raster
        self.coordinate_system = CoordinateSystemValues.EPSG_4326
        self.data_type = RasterDataTypeValues.Byte
        self.no_data = 255
        self.file_type = FileTypeValues.GeoTIFF
        self.compression = CompressionValues.LZW
        self.product_type = ProductTypeValues.CropClassification
        self.classification = dict()


def main():
    return CropClassificationModel()


class CropClassificationDataProperties(DataProperties):
    def need_data_value_list(self):
        return True


def main_properties():
    return CropClassificationDataProperties()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-15


from gagospecs.v1.base.validator import Validator, \
    DataProperties, \
    CoordinateSystemFieldValidator
from gagospecs.v1.models.original.NDVI import NDVIModel
from gagospecs.v1.schemas.original.NDVI import NDVISchema


class NDVIValidator(Validator):
    """
    原始级NDVI产品校验
    """
    def __init__(self):
        super().__init__(
            'NDVIValidator',
            [CoordinateSystemFieldValidator()]
        )

    def _validate_data(
            self,
            data_properties: DataProperties,
            gmd: dict
    ) -> dict:
        ndvi_model: NDVIModel = NDVIModel()
        ndvi_model.data_type = data_properties.data_type
        ndvi_model.lines = data_properties.lines
        ndvi_model.rows = data_properties.rows
        ndvi_model.no_data = data_properties.no_data
        ndvi_model.compression = data_properties.compression
        ndvi_model.coordinate_system = data_properties.coordinate_system
        ndvi_model.extent = data_properties.extent
        ndvi_schema: NDVISchema = NDVISchema()
        _, errors = ndvi_schema.dumps(ndvi_model)
        return errors


def main():
    return NDVIValidator()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-07


from logging import Logger, getLogger


class SpecBase:
    def __init__(
            self,
            name: str
    ):
        self.name: str = name
        self.logger: Logger = getLogger(self.name)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-08


from datetime import datetime

from gagospecs.v1.base.load_module import create_module


class DataProperties:
    """
    用于检验数据本体属性与元数据的集合
    """
    def __init__(self):
        # file type, FileTypeValues
        self.file_type = ''
        # RasterDataTypeValues
        self.data_type: str = ''
        # no data value
        self.no_data: int = -1
        # line count
        self.lines: int = -1
        # row count
        self.rows: int = -1
        # resolution of image
        self.resolution: float = -1
        # CompressionValues, NA, LZW
        self.compression: str = ''
        # 4326 3587....
        self.coordinate_system: float = -1.0
        # [[x1, y1], [x1, y2], [x2, y2], [x2, y1]]
        self.extent: str = ''
        # enum of data values [1, 3, 5, 6]
        # for crop classification like product
        self.data_value_list: list = []
        # min value, for continuous product
        self.data_min: int = None
        # max value, for continuous product
        self.data_max: int = None
        # GeometryTypeValues, for Vector Product
        self.geometry_type: str = ''

    def need_data_value_list(self) -> bool:
        return False

    def need_data_min(self) -> bool:
        return False

    def need_data_max(self) -> bool:
        return False


class GMDModel:
    def __init__(self):
        self.version: str = '1.0'
        self.name: str = ''
        self.description: str = ''
        self.level: str = ''
        self.category: str = ''
        self.product_type: str = ''
        self.file_type: str = ''
        self.production_date: datetime = None
        self.producer: str = ''
        self.qc: str = ''

    def validate(
            self,
            data_properties: DataProperties
    ) -> (bool, list):
        validation_errors: list = []
        if self.file_type != data_properties.file_type:
            validation_errors.append('file_type')
        self._validate(
            data_properties,
            validation_errors
        )
        return len(validation_errors) == 0, validation_errors

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        pass


class SpatialGMDModel(GMDModel):
    def __init__(self):
        super().__init__()
        self.coordinate_system: str = ''
        self.extent: str = ''
        self.bias: str = 'NA'

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.coordinate_system != data_properties.coordinate_system:
            errors.append('coordinate_system')
        if self.extent != data_properties.extent:
            errors.append('extent')


class RasterGMDModel(SpatialGMDModel):
    def __init__(self):
        super().__init__()
        self.category = CategoryValues.Raster
        self.no_data: int = -1
        self.data_type: str = None
        self.lines: int = 0
        self.rows: int = 0
        self.resolution: float = -1

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.no_data != data_properties.no_data:
            errors.append('no_data')
        if self.data_type != data_properties.data_type:
            errors.append('data_type')
        if self.lines != data_properties.lines:
            errors.append('lines')
        if self.rows != data_properties.rows:
            errors.append('rows')
        if self.resolution != data_properties.resolution:
            errors.append('resolution')


class FloatRasterGMDModel(RasterGMDModel):
    def __init__(self):
        super().__init__()
        self.scale_factor: float = 1.0
        self.linear_factor: float = 1.0


class GeoTIFFRasterGMDModel(GMDModel):
    def __init__(self):
        self.compression: str = None
        super().__init__()

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.compression != data_properties.compression:
            errors.append('compression')


def create_model(
        product_level: str,
        product_type: str
) -> GMDModel or None:
    """
    Create a GMD model
    :param product_level: product level,
    which laid in gagospecs.v1.base.product_level.ProductLevel class
    :param product_type: product name
    :return: requested GMD model object
    """
    m = create_module(
        'models',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main'):
            return getattr(m, 'main')()
    return None


def create_data_properties(
        product_level: str,
        product_type: str
) -> DataProperties or None:
    """
    Get DataProperies object needed
    for the specified product
    :param product_level:
    :param product_type:
    :return:
    """
    m = create_module(
        'models',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main_properties'):
            return m.main_properties()
    return DataProperties()


class ValuesBase:
    """
    值域表基类
    """
    @classmethod
    def get_value_set(cls) -> set:
        values_set: set = set()
        for k, v in cls.__dict__.items():
            key: str = k
            value: str = v
            if (not key.startswith('__')
                and value not in values_set):
                values_set.add(value)
        return values_set


class ProductLevelValues(ValuesBase):
    """
    Product level
    """
    Original = 'original'
    Publishable = 'publishable'
    deliverable = 'deliverable'


class CategoryValues(ValuesBase):
    """
    产品分类值域表
    """
    Raster = 'Raster'
    Vector = 'Vector'
    Attribute = 'Attribute'


class ProductTypeValues(ValuesBase):
    """
    产品类型值域表
    """
    BaseMap = 'BaseMap'
    NDVI = 'NDVI'
    CropClassification = 'CropClassification'
    DisasterAssessment = 'DisasterAssessment'
    PDSI = 'PDSI'
    SMAP = 'SMAP'
    AdministrativeArea = 'AdministrativeArea'
    LandBlock = 'LandBlock'
    Statistics = 'Statistics'
    WeatherLive = 'WeatherLive'
    WeatherHistory = 'WeatherHistory'
    WeatherForecast = 'WeatherForecast'


class WeatherProductTypeValues(ValuesBase):
    """
    气象产品类型值域表
    """
    NA = 'NA'
    WeatherPhenomenon = 'WeatherPhenomenon'
    Temperature = 'Temperature'
    Moisture = 'Moisture'
    Precipitation = 'Precipitation'
    WindDirection = 'WindDirection'
    WindPower = 'WindPower'
    Pressure = 'Pressure'
    AirQuality = 'AirQuality'


class FileTypeValues(ValuesBase):
    """
    文件类型值域表
    """
    GeoTIFF = 'GeoTIFF'
    JPEG = 'JPEG'
    PNG = 'PNG'
    LERC = 'LERC'
    BIP = 'BIP'
    GeoJSON = 'GeoJSON'
    SHP = 'SHP'
    JSON = 'JSON'


class CoordinateSystemValues(ValuesBase):
    """
    坐标系统值域表
    """
    EPSG_3857 = 'EPSG:3857'
    EPSG_4326 = 'EPSG:4326'


class BiasValues(ValuesBase):
    """
    坐标系偏移值域表
    """
    NA = 'NA'
    GCJ_02 = 'GCJ-02'


class RasterDataTypeValues(ValuesBase):
    """
    栅格像元数据类型值域表
    """
    Byte = 'Byte'
    UInt16 = 'UInt16'
    UInt32 = 'UInt32'


class CompressionValues(ValuesBase):
    """
    栅格数据压缩方法值域表
    """
    NA = 'NA'
    LZW = 'LZW'


class GeometryTypeValues(ValuesBase):
    """
    几何类型值域表
    """
    Point = 'Point'
    Polyline = 'Polyline'
    Polygon = 'Polygon'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-17


from gagospecs.v1.base.model import DataProperties
from gagospecs.v1.base.validator import Validator, \
    CoordinateSystemFieldValidator
from gagospecs.v1.models.original.CropClassification \
    import CropClassificationModel
from gagospecs.v1.schemas.original.CropClassification \
    import CropClassificationSchema


class CropClassificationValidator(Validator):
    """
    Crop classification validator
    """
    def __init__(self):
        super().__init__(
            'CropClassificationValidator',
            [CoordinateSystemFieldValidator()]
        )

    def _validate_data(
            self,
            data_properties: DataProperties,
            gmd: dict
    ) -> dict:
        crop_classification_model: CropClassificationModel = \
            CropClassificationModel()
        crop_classification_model.data_type = \
            data_properties.data_type
        crop_classification_model.lines = \
            data_properties.lines
        crop_classification_model.rows = \
            data_properties.rows
        crop_classification_model.no_data = \
            data_properties.no_data
        crop_classification_model.compression = \
            data_properties.compression
        crop_classification_model.coordinate_system = \
            data_properties.coordinate_system
        crop_classification_model.extent = \
            data_properties.extent
        crop_classification_schema: CropClassificationSchema = \
            CropClassificationSchema()
        _, errors = \
            crop_classification_schema.dumps(crop_classification_model)

        return errors


def main():
    return CropClassificationValidator()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-17


FIELD_MAPPINGS = {
    'file_type': ['文件类型', ''],
    'no_data': ['空像素', ''],
    'coordinate_system': ['坐标系统', ''],
    'extent': ['坐标范围', ''],
    'bias': ['坐标偏移算法', ''],
    'data_type': ['像元数据类型', ''],
    'lines': ['栅格数据列数', ''],
    'rows': ['栅格数据行数', ''],
    'resolution': ['分辨率', ''],
    'compression': ['压缩算法', ''],
    'geometry_type': ['几何类型', ''],
}



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-07


from gagospecs.v1.base.converter import Converter


class ClassificationConverter(Converter):
    def __init__(self):
        super().__init__('classification')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018--10-18


from gagospecs.base.base import SpecBase
from gagospecs.v1.base.load_module import create_module
from gagospecs.v1.base.model import GMDModel


class GMDGenerator(SpecBase):
    """
    Gago meta data(.gmd) generator base class
    """

    def generate(
            self,
            gmd_model: GMDModel
    ) -> str:
        """
        generate gago meta data str from a model
        :param gmd_model:
        :return:
        """
        pass


def create_generator(
        product_level: str,
        product_type: str,
) -> GMDGenerator or None:
    """
    Create a GMD generator
    :param product_level:
    :param product_type:
    :return:
    """
    m = create_module(
        'generators',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main'):
            return m.main()
    return None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-15


import struct

from osgeo import gdal

from gagospecs.v1.base.converter import Converter


class NDVIConverter(Converter):
    def __init__(self):
        super().__init__('ndvi')

    def convert(
            self,
            src_ndvi_tiff_path: str,
            dst_ndvi_tiff_path: str
    ):
        """
        按照佳格NDVI原始级数据产品规范要求，转换一幅普通NDVI产品
        :param src_ndvi_tiff_path: 源NDVI栅格数据，按照常用数据类型存放
        :param dst_ndvi_tiff_path: 目标NDVI栅格数据，按照佳格NDVI规范存放
        :return: None
        """
        gdal.AllRegister()

        src_ds: gdal.Dataset = gdal.Open(
            src_ndvi_tiff_path,
            gdal.OF_READONLY
        )
        tiff_driver: gdal.Driver = gdal.GetDriverByName('GTiff')
        dst_ds: gdal.Dataset = tiff_driver.Create(
            dst_ndvi_tiff_path,
            src_ds.RasterXSize,
            src_ds.RasterYSize,
            1,
            gdal.GDT_Byte
        )
        dst_ds.SetProjection(src_ds.GetProjection())
        dst_ds.SetGeoTransform(src_ds.GetGeoTransform())

        src_band: gdal.Band = src_ds.GetRasterBand(1)
        src_data_type = src_band.DataType
        src_data_type_name = gdal.GetDataTypeName(src_data_type)
        src_data_length = int(gdal.GetDataTypeSize(src_data_type) / 8)
        dst_band: gdal.Band = dst_ds.GetRasterBand(1)

        byte_count_per_row = src_data_length * src_band.XSize
        for row_index in range(src_band.YSize):
            data: bytes = src_band.ReadRaster(
                0,
                row_index,
                src_band.XSize,
                1
            )
            l: int = len(data)
            start_index: int = 0
            dst_data_list: list = list()
            while True:
                src_ndvi_value, = struct.unpack(
                    'f',
                    data[start_index:start_index + src_data_length]
                )
                src_ndvi_value: float = src_ndvi_value
                if src_ndvi_value > 1 or src_ndvi_value < -1:
                    print('invalid ndvi value: %f' % src_ndvi_value)
                dst_ndvi_value: int = int(src_ndvi_value * 127 + 126)
                dst_data_list.append(dst_ndvi_value)
                start_index += src_data_length
                if start_index >= byte_count_per_row:
                    break

            dst_data = struct.pack(
                '%dB' % len(dst_data_list),
                *dst_data_list
            )
            dst_ds.WriteRaster(
                0,
                row_index,
                src_band.XSize,
                1,
                dst_data
            )



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from distutils.core import setup

setup(
    name='gagospecs',
    version='0.2.0',
    description='Gago Specs Lib',
    author='Dajiang Ren',
    author_email='rendajiang@gagogroup.cn',
    url='http://www.gagogroup.cn',
    packages=[
        'gagospecs',
        'gagospecs.base',
        'gagospecs.v1',
        'gagospecs.v1.base',
        'gagospecs.v1.converters',
        'gagospecs.v1.generators',
        'gagospecs.v1.generators.original',
        'gagospecs.v1.models',
        'gagospecs.v1.models.original',
        'gagospecs.v1.converters',
        'gagospecs.v1.converters.original',
        'gagospecs.v1.schemas',
        'gagospecs.v1.schemas.original',
        'gagospecs.v1.services',
        'gagospecs.v1.validators',
        'gagospecs.v1.validators.original',
    ],
    requires=[
        'gdal',
        'marshmallow'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
